package com.joaquim.calvo.serloc.serlocapi.application.service;

import com.joaquim.calvo.serloc.serlocapi.application.service.persistance.SerlocServiceRepository;
import com.joaquim.calvo.serloc.serlocapi.application.service.service.SerlocServiceImpl;
import com.joaquim.calvo.serloc.serlocapi.domain.service.ServiceType;
import com.joaquim.calvo.serloc.serlocapi.domain.service.request.CreateNewServiceRequest;
import com.joaquim.calvo.serloc.serlocapi.domain.service.request.GetServicesRequest;
import com.joaquim.calvo.serloc.serlocapi.domain.service.service.ServiceTypeService;
import com.joaquim.calvo.serloc.serlocapi.domain.user.User;
import com.joaquim.calvo.serloc.serlocapi.domain.user.UserType;
import com.joaquim.calvo.serloc.serlocapi.domain.user.service.UserService;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import java.util.Optional;

public class SerlocServiceImplTest {

    @InjectMocks
    private SerlocServiceImpl serlocService;

    @Mock
    private SerlocServiceRepository serlocServiceRepository;

    @Mock
    private UserService userService;

    @Mock
    private ServiceTypeService serviceTypeService;

    @Before
    public void init () {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void createNewServiceMustGetUserInformation() {
        mockAllCalls();
        final CreateNewServiceRequest request = getDefaultRequest();

        serlocService.createNewService(request);

        Mockito.verify(userService, Mockito.times(1)).getUserByEmail(request.getServiceProviderEmail());
    }

    @Test
    public void createNewServiceMustGetServiceTypeForTheNewService() {

        mockAllCalls();
        final CreateNewServiceRequest request = getDefaultRequest();

        serlocService.createNewService(request);

        Mockito.verify(serviceTypeService, Mockito.times(1)).getServiceTypeById(request.getServiceTypeId());
    }

    @Test
    public void createNewServiceMustGetANewServiceId() {
        mockAllCalls();
        final CreateNewServiceRequest request = getDefaultRequest();

        serlocService.createNewService(request);

        Mockito.verify(serlocServiceRepository, Mockito.times(1)).getNewId();
    }

    @Test
    public void createNewServiceMustPersistTheGeneratedService() {
        mockAllCalls();
        final CreateNewServiceRequest request = getDefaultRequest();

        serlocService.createNewService(request);

        Mockito.verify(serlocServiceRepository, Mockito.times(1)).saveNewService(Mockito.any());
    }

    @Test
    public void getAvailableServicesMustCallRepository() {
        final GetServicesRequest request = new GetServicesRequest();

        serlocService.getAvailableServices(request);

        Mockito.verify(serlocServiceRepository, Mockito.times(1)).getAvailableServices(request);
    }

    @Test
    public void getServicesForUserByEmailMustCallRepository() {
        final String userEmail = "testUser@testuser.com";

        serlocService.getServicesForUserByEmail(userEmail);

        Mockito.verify(serlocServiceRepository, Mockito.times(1)).getUserServices(userEmail);
    }

    /**
     * Obtain a default request for the service.
     * @return default request.
     */
    private CreateNewServiceRequest getDefaultRequest() {
        final CreateNewServiceRequest request = new CreateNewServiceRequest();

        request.setDescription("description");
        request.setLatitude(22.1F);
        request.setLongitude(22.1F);
        request.setName("new service name");
        request.setPrice(12);
        request.setServiceProviderEmail("testuser@test.com");
        request.setServiceTypeId(1);

        return request;
    }

    /**
     * Method which set behaviour to all the mocked injected services and components.
     */
    private void mockAllCalls() {
        final UserType mockedUserType = new UserType(1, "new User type");
        final User returnedUser = new User("mockedUserEmail", "mockedUserName", "i48r3rf3r4f34f", mockedUserType,
                "633651614");
        Mockito.when(userService.getUserByEmail(Mockito.anyString())).thenReturn(Optional.of(returnedUser));

        final ServiceType returnedServiceType = new ServiceType(1, "retured service type", "returned service description");
        Mockito.when(serviceTypeService.getServiceTypeById(Mockito.anyInt())).thenReturn(returnedServiceType);

        Mockito.when(serlocServiceRepository.getNewId()).thenReturn(1);
    }

}
